import glob from 'glob';
import curry from 'crocks/helpers/curry';
import chalk from 'chalk';
import deepmerge from 'deepmerge';
import {
  existsSync,
  lstatSync,
  mkdirSync,
  readFileSync,
  writeFileSync,
} from 'fs';
import { join, dirname } from 'path';

const { green } = chalk;

export const virtualFiles = [];

export function resetVirtualFiles() {
  virtualFiles.length = 0;
}

/**
 * changes dash-case tag-name to PascalCase ClassName
 * @param  {String} dashCase tag name
 * @return {String}          class name
 */
export function dashToPascalCase(dashCase) {
  return dashCase
    .split('-')
    .reduce((previous, part) => previous + part.charAt(0).toUpperCase() + part.slice(1), '');
}

/**
 * Reduces a data entry onto a partial template result.
 * @param  {String} partial partial template result
 * @param  {[key, value]} entry data entry e.g. [tagName, 'fc-foo']
 * @return {String}         partial template result
 */
const reduceEntry = (partial, [key, value]) => {
  partial = partial.replace(new RegExp(`<%= ${key} %>`, 'g'), value || '');
  return partial;
};

/**
 * Minimal templating system.
 * Replaces occurrences of `<%= name %>` in template with provided value
 *
 * @type {Function}
 * @example
 * processTemplate('prefix <%= name %> suffix', { name: 'foo' })
 * // prefix foo suffix
 *
 * @param {string} template Template file contents
 * @param {object} data Object mapping variable name to string
 * @return {string} generated file contents with interpolated values
 */
export const processTemplate = (template, data = {}) =>
  Object.entries(data)
    .reduce(reduceEntry, template);


/**
 * Minimal virtual file system
 * Stores files to write in an array
 *
 * @param {string} filePath
 * @param {string} content
 */
export function writeFileToPath(filePath, content) {
  let addNewFile = true;
  virtualFiles.forEach((fileMeta, index) => {
    if (fileMeta.path === filePath) {
      virtualFiles[index].content = content;
      addNewFile = false;
    }
  });
  if (addNewFile === true) {
    virtualFiles.push({ path: filePath, content });
  }
}

/**
 *
 * @param {string} filePath
 * @return {any}
 */
export function readFileFromPath(filePath) {
  let content = false;
  virtualFiles.forEach((fileMeta, index) => {
    if (fileMeta.path === filePath) {
      // eslint-disable-next-line prefer-destructuring
      content = virtualFiles[index].content;
    }
  });
  if (content) {
    return content;
  }
  if (existsSync(filePath)) {
    return readFileSync(filePath, 'utf-8');
  }
  return false;
}

/**
 *
 * @param {string} filePath
 */
export function deleteVirtualFile(filePath) {
  const index = virtualFiles.findIndex(fileMeta => fileMeta.path === filePath);
  if (index !== -1) {
    virtualFiles.splice(index, 1);
  }
}

/**
 *
 * @param {string} toPath
 * @param {string} fileContent
 */
export function writeFileToPathOnDisk(toPath, fileContent) {
  const toPathDir = dirname(toPath);
  if (!existsSync(toPathDir)) {
    mkdirSync(toPathDir, { recursive: true });
  }
  writeFileSync(toPath, fileContent);
  console.log(green`  👉 Wrote ` + toPath);
}

/**
 * Actually writes the virtual files to the real file system.
 */
export function writeFilesToDisk() {
  virtualFiles.forEach(fileMeta => {
    writeFileToPathOnDisk(fileMeta.path, fileMeta.content);
  });
}

/**
 * Processes a template with data, saves the results to the virtual file system.
 * @param {string} fromPath path to template file.
 * @param {string} toPath   destination path in virtual file system
 * @param {object} data     template interpolation data.
 */
export const copyTemplate = curry(function _copyTemplate(data, fromPath, toPath) {
  const fileContent = readFileFromPath(fromPath);
  if (fileContent) {
    const processed = processTemplate(fileContent, data);
    writeFileToPath(toPath, processed);
  }
});

/**
 *
 * @param {string} fromGlob
 * @param {string} [toDir] Directory to copy into
 * @param {object} data Replace parameters in files
 * @return {Promise}
 */
export function copyTemplates(fromGlob, toDir = process.cwd(), data = {}) {
  return new Promise(resolve => {
    glob(fromGlob, { dot: true }, (er, files) => {
      files.forEach(filePath => {
        if (!lstatSync(filePath).isDirectory()) {
          const fileContent = readFileFromPath(filePath);
          if (fileContent !== false) {
            const processed = processTemplate(fileContent, data);

            // find path write to (force / also on windows)
            const replace = join(fromGlob.replace(/\*/g, '')).replace(/\\(?! )/g, '/');
            const toPath = filePath.replace(replace, `${toDir}/`);

            writeFileToPath(toPath, processed);
          }
        }
      });
      resolve();
    });
  });
}

/**
 * Processes a json file template and data.
 * See: copyTemplate
 * @param {string} fromPath path to template file.
 * @param {string} toPath   destination path in virtual file system
 * @param {object} data     template interpolation data.
 */
export const copyTemplateJsonInto = curry(function _copyTemplateJsonInto(data, fromPath, toPath) {
  const content = readFileFromPath(fromPath);
  if (content === false) {
    return;
  }
  const processed = processTemplate(content, data || {});
  const mergeMeObj = JSON.parse(processed);

  const overwriteMerge = (destinationArray, sourceArray) => sourceArray;

  let finalObj = mergeMeObj;
  const sourceContent = readFileFromPath(toPath);
  if (sourceContent) {
    finalObj = deepmerge(JSON.parse(sourceContent), finalObj, { arrayMerge: overwriteMerge });
  }

  writeFileToPath(toPath, JSON.stringify(finalObj, null, 2));
});
