import { join } from 'path';
import {
  copyTemplate,
  copyTemplateJsonInto,
  dashToPascalCase,
  writeFilesToDisk,
} from './core';

const copy = data => ({ template, destination }) =>
  (template.endsWith('json') ? copyTemplateJsonInto : copyTemplate)(
    data,
    `${__dirname}/templates/${data.elementType}/${template}`,
    join(process.cwd(), `packages/${data.tagName}/${destination}`)
  );

/**
 * Executes the generator.
 * Generates and writes
 *  - src/index.js
 *  - src/ForterEl.js
 *  - src/forter-el.css
 *  - src/forter-el.js
 *  - src/forter-el.spec.js
 *  - package.json
 * @param  {String} tagName  dash-case element tag name
 * @param  {String} authorEmail author's email
 * @param  {String} authorName  author's name
 */
export async function execute({ tagName, authorEmail, authorName, elementType }) {
  /**
   * Name of the class
   * @type {String}
   */
  const className = dashToPascalCase(tagName);

  /**
   * Name of the element. e.g. for <forter-input>, 'input'
   * Used to generate the package name i.e. `@forter/input`
   * @type {String}
   */
  const elementName = tagName.replace(/^forter-/, '');

  /**
   * Name of the element humanize. e.g. for <forter-input>, 'Input'
   * Used to generate the story name i.e. `Input`
   * @type {String}
   */
  const name = className.replace('Forter', '');

  /**
   * Maps template files to destination files
   * @type {{template: string, destination: string}[]}
   */
  const files = [
    { template: `_gitignore`, destination: `.gitignore` },
    { template: `_README.md`, destination: `README.md` },
    { template: `_HOWTO.md`, destination: `HOWTO.md` },
    { template: `_web-components.json`, destination: `web-components.json` },
    { template: `_LICENSE.md`, destination: `LICENSE.md` },
    { template: `_index.js`, destination: `/src/index.js` },
    { template: `_ForterEl.js`, destination: `/src/${className}.js` },
    { template: `_forter-el.css`, destination: `/src/${tagName}.css` },
    { template: `_forter-el.js`, destination: `/src/${tagName}.js` },
    { template: `_forter-el.spec.js`, destination: `/src/${tagName}.spec.js` },
    { template: `_stories/_forter-el.stories.js`, destination: `/stories/${tagName}.stories.js` },
    { template: `_stories/_disabled.story.js`, destination: `/stories/disabled.story.js` },
    { template: `_package.json`, destination: `/package.json` },
    { template: `_test/_when disabled/_expected.html`, destination: `/test/when disabled/expected.html` },
    { template: `_test/_when disabled/_input.json`, destination: `/test/when disabled/input.json` },
    { template: `_test/_when disabled/_template.html`, destination: `/test/when disabled/template.html` },
    { template: `_test/_when enabled/_expected.html`, destination: `/test/when enabled/expected.html` },
    { template: `_test/_when enabled/_input.json`, destination: `/test/when enabled/input.json` },
    { template: `_test/_when enabled/_template.html`, destination: `/test/when enabled/template.html` },
  ];

  const { version: coreVersion } =
    require('../../lib/core/package.json');
  const { version: componentBaseVersion } =
      require('../../packages/component-base/package.json');
  const { version: layoutVersion } =
      require('../../packages/layout/package.json');

  // Generate 'virtual files'
  files.forEach(copy({
    authorEmail,
    authorName,
    elementType,
    className,
    elementName,
    name,
    coreVersion,
    componentBaseVersion,
    layoutVersion,
    tagName,
  }));

  // Write those 'virtual files' to disk
  await writeFilesToDisk();

  console.log('All done! 😎');
}
