import chalk from 'chalk';
import commandLineArgs from 'command-line-args';
import prompts from 'prompts';
import { getGitConfig } from './git-config';

const { blue } = chalk;

prompts.override(commandLineArgs([
  { name: 'tagName', type: String },
]));

/**
 * Pretty banner that displays when running the cli
 * @type {String}
 */
const header = blue`


  /yyyyyyyyyyyy:   .-:///::+p   .yyyyyyyyysoo. -yyyyyyyyyyyyyy oyyyyyyyyyyyy-.yyyyyyyyyso/.
  oyyyyyyyyyyyy.  .::::--/o/.o  :yyyyyyyyyyyyy:/yyyyyyyyyyyyyo yyyyyssssssss :yyyyyyyyyyyyy:
  yyyy/          :----.:s+ oo / +yyyo    .yyyy+     syyy+     .yyyy.         +yyyo    .syyyo
 .yyyyo++++++/  :////+s/ s/.y   syyy+   .:yyyy-     yyyy:     :yyyysssssss+  syyy+   .:yyyy-
 /yyyyyyyyyyyo .-:::+/.y o/.y  .yyyyyyyyyyyyo-     -yyyy.     +yyyysssssss/ .yyyyyyyyyyyyo-
 oyyyo.......  .+.o-///+.s //  :yyyy++syyyy-       /yyyy      syyy/         :yyyy++syyyy-
 syyy/         -o/:::+/-o.-+   +yyys   syyys.      oyyyo     .yyyysoooooooo +yyys   syyys.
.yyyy-            :/:.// -    syyy+    oyyyy-     yyyy/     :yyyyyyyyyyyys syyy+    oyyyy-


`;

/**
 * Ensures that tag names start with `forter-` and only contain alphanumerics or `-`
 * @param  {String} tagName tag name
 * @return {Boolean|String}        true when tag name is valid, error message otherwise
 */
const validateTagName = tagName =>
  /^(fc|forter)(?=.*-)[a-z0-9-]+$/
    .test(tagName) ? true : 'name must start with "fc-" or "forter-", e.g. "forter-foot-ball"';

const validateType = type =>
  /forter-element|lit-element/
    .test(type) ? true : `must have either lit-element or forter-element as type`;
/**
 * Prompts the user for information about their element
 * @return {Object} object mapping prompt `name` property to user-supplied value
 */
export async function prompt() {
  console.log(header);
  return prompts([{
    type: 'text',
    name: 'tagName',
    message: 'What is the tag name for your web component?',
    initial: 'forter-foo',
    validate: validateTagName,
  }, {
    type: 'text',
    name: 'elementType',
    message: 'What is the type of component lit-element or forter-element?',
    initial: 'forter-element',
    validate: validateType,
  }, {
    type: 'text',
    name: 'authorName',
    message: 'What is your name (as a contributor in package.json)?',
    initial: await getGitConfig('user.name'),
  }, {
    type: 'text',
    name: 'authorEmail',
    message: 'What is your email (as a contributor in package.json)?',
    initial: await getGitConfig('user.email'),
  },
  ]);
}
